/* FILE: functions.h                    -*-Mode: c++-*-
 *
 * Non-class C++ functions defined by the Nb extension.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 2000/10/03 19:55:30 $
 * Last modified by: $Author: dgp $
 */

#ifndef _NB_FUNCTIONS
#define _NB_FUNCTIONS

#include "dstring.h"

/* End includes */     /* Optional directive to build.tcl */

// Greatest Common Divisor
int Nb_Gcd(int m,int n);
Tcl_CmdProc NbGcdCmd;

// Simple continued fraction like rational approximator
void Nb_RatApprox(double x,int steps,int &num,int &denom);
Tcl_CmdProc NbRatApprox;

// Version of atof that allows 'D' or 'd' to be used to denote exponent.
double Nb_Atof(const char *nptr);

// Portable case insensitive string comparison, modeled after
// BSD's strcasecmp().
int Nb_StrCaseCmp(const char *s1, const char *s2);

// Routines to detect IEEE floating point NAN's and infinities.
int Nb_IsFinite(REAL4 x);
int Nb_IsFinite(REAL8 x);

// C interface to the Tcl 'file nativename' functionality.
void Nb_TclNativeFilename(Tcl_Interp* interp,Nb_DString &filename);
inline void Nb_TclNativeFilename(Nb_DString &filename) {
  Nb_TclNativeFilename((Tcl_Interp*)NULL,filename);
}

// A version of fopen that calls Nb_TclNativeFilename().
FILE *Nb_FOpen(const char *path,const char *mode);

//////////////////////////////////////////////////////////////////////////
// Routines to interchange two values
template<class T> void swap(T &x,T &y);  // Explicit, separate declaration
/// to help HP CC auto-instantiation.

// Include definition in header to aid GCC's auto-instantiation.
template<class T> void swap(T &x,T &y) {
  T temp=x; x=y; y=temp;
}

#endif /* _NB_FUNCTIONS */
