/* FILE: imports.cc              -*-Mode: c++-*-
 *
 *	Replacement implementations for missing Tcl/Tk APIs
 * in old Tcl/Tk releases.
 *
 * Last modified on: $Date: 2000/10/17 20:33:47 $
 * Last modified by: $Author: dgp $
 */

#include "autobuf.h"
#include "imports.h"
#include "nullchannel.h"
/* End includes */     

/* Domain checked atan2 function */
double Oc_Atan2(double y,double x)
{
  // The following #define trick allows atan2 to be defined
  // to be Oc_Atan2 in port.h, but allows us to call the system
  // atan2 function from inside this routine.  When the atan2 call
  // is processed by the compiler, if it has previously been
  // #define'd to be Oc_Atan2, then the macro #define'd here kicks
  // in and sets it back to atan2.  Since atan2 is the name of the
  // original macro that is being expanded, macro expansion stops
  // (as per Section 16.3 (?) of the ARM).  If atan2 was not
  // #define'd, or if it was #define'd to something other than
  // Oc_Atan2, then the macro here has no effect.
#define Oc_Atan2(y,x) atan2((y),(x))
  if(x==0.0 && y==0.0) return 0.0;
  return atan2(y,x);
#undef Oc_Atan2
}

#if (TCL_MAJOR_VERSION < 8)
int
Tcl_GetChannelHandle(Tcl_Channel channel, int dir, ClientData *handlePtr)
{
    Tcl_File tfile=Tcl_GetChannelFile(channel,dir);
    if(tfile==NULL) return TCL_ERROR;
    *handlePtr=Tcl_GetFileInfo(tfile,NULL);
    return TCL_OK;
}
#endif

#if ((TCL_MAJOR_VERSION < 8) || ((TCL_MAJOR_VERSION == 8) \
    && (TCL_MINOR_VERSION == 0) && (TCL_RELEASE_SERIAL < 6)))
char *
Tcl_PkgPresent(Tcl_Interp* interp, char* name, char* version, int exact)
{
    // Need Oc_AutoBuf to support Tcl 7.  
    // Starting with Tcl 8.0p0, the interface is CONST
    Oc_AutoBuf ab;
    Tcl_DString buf;
    Tcl_DString loadedVersion;

    /*
     * Use [package provide] to query if the named package is
     * already in the interpreter.
     */ 

    Tcl_DStringInit(&buf);
    Tcl_DStringAppend(&buf, ab("package provide"), -1);
    Tcl_Eval(interp, Tcl_DStringAppendElement(&buf, name));
    Tcl_DStringFree(&buf);

    Tcl_DStringInit(&loadedVersion);
    Tcl_DStringGetResult(interp, &loadedVersion);
    if ( !(Tcl_DStringLength(&loadedVersion)) ) {
        Tcl_DStringFree(&loadedVersion);
        if (version != NULL) {
            Tcl_AppendResult(interp, "package ", name, " ", version,
                    " is not present", (char *) NULL);
        } else {
            Tcl_AppendResult(interp, "package ", name, 
                    " is not present", (char *) NULL);
        }
        return NULL;
    }
    Tcl_DStringFree(&loadedVersion);

    /*
     * At this point we know some version is loaded, so 
     * Tcl_PkgRequire will have no side effect.  Call it
     * to do the rest of the work.
     */

    return Tcl_PkgRequire(interp, name, version, exact);
}
#endif

#if (TK_MAJOR_VERSION < 8) \
        || ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION < 2))
void
Tk_InitConsoleChannels(Tcl_Interp* interp)
{

/*
 * In Tcl 8.x, a set of null channels can replace the standard channels
 * to prevent EOF on stdin, or other problems from terminating the shell
 * too soon.  
 *
 * We have not yet implemented a "real" Tcl 7.x version of the null
 * channels, so that won't work.  If we ever do, remove this restriction.
 * For now, EOF on stdin will kill shells build with Tcl 7.
 */
#if (TCL_MAJOR_VERSION > 7)
    Tcl_Channel channel;
    Oc_AutoBuf ab1, ab2;

    channel = Nullchannel_Open();
    if (channel != NULL) {
        Tcl_SetChannelOption(NULL, channel, ab1("-translation"), ab2("lf"));
        Tcl_SetChannelOption(NULL, channel, ab1("-buffering"), ab2("none"));
#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION == 1)
	Tcl_SetChannelOption(NULL, channel, ab1("-encoding"), ab2("utf-8"));
#endif
    }
    Tcl_SetStdChannel(channel, TCL_STDIN);
    Tcl_RegisterChannel(interp, channel);

    channel = Nullchannel_Open();
    if (channel != NULL) {
        Tcl_SetChannelOption(NULL, channel, ab1("-translation"), ab2("lf"));
        Tcl_SetChannelOption(NULL, channel, ab1("-buffering"), ab2("none"));
#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION == 1)
	Tcl_SetChannelOption(NULL, channel, ab1("-encoding"), ab2("utf-8"));
#endif
    }
    Tcl_SetStdChannel(channel, TCL_STDOUT);
    Tcl_RegisterChannel(interp, channel);

    channel = Nullchannel_Open();
    if (channel != NULL) {
        Tcl_SetChannelOption(NULL, channel, ab1("-translation"), ab2("lf"));
        Tcl_SetChannelOption(NULL, channel, ab1("-buffering"), ab2("none"));
#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION == 1)
	Tcl_SetChannelOption(NULL, channel, ab1("-encoding"), ab2("utf-8"));
#endif
    }
    Tcl_SetStdChannel(channel, TCL_STDERR);
    Tcl_RegisterChannel(interp, channel);
#endif

}
#endif

